@@ -3,7 +3,6 @@
 #require "topkg"
 open Topkg
 
-
 let uutf = Conf.with_pkg "uutf"
 let cmdliner = Conf.with_pkg "cmdliner"
 
@@ -13,7 +12,5 @@ let () =
   let cmdliner = Conf.value c cmdliner in
   Ok [ Pkg.mllib ~api:["Uuseg"; "Uuseg_string"] "src/uuseg.mllib";
        Pkg.bin ~cond:(uutf && cmdliner) "test/usegtrip";
-       Pkg.test "test/test";
-       Pkg.test "test/examples";
        Pkg.doc "doc/index.mld" ~dst:"odoc-pages/index.mld";
        Pkg.doc "test/examples.ml"; ]
